use std::{
	convert::Infallible,
	error::Error,
	sync::Arc,
	time::Duration,
};

use hyper::{
	StatusCode,
	Uri,
};
use reqwest::{
	Body,
	Client,
};
use tokio::{
	fs::File,
	io::AsyncReadExt,
	sync::mpsc::{
		channel,
		Receiver,
	},
	time::delay_for,
};

#[tokio::main]
async fn main () -> Result <(), Box <dyn Error>> {
	let client = Arc::new (Client::new ());
	
	let mut backoff_delay = 0;
	
	loop {
		if backoff_delay > 0 {
			delay_for (Duration::from_millis (backoff_delay)).await;
		}
		
		let _uri = Uri::builder ()
			.scheme ("http")
			.authority ("127.0.0.1:4000")
			.path_and_query ("/http_listen/alien_wildlands")
		.build ().unwrap ();
		
		let req_req = client.get ("http://127.0.0.1:4000/http_listen/alien_wildlands");
		
		println! ("Step 1");
		let req_resp = match req_req.send ().await {
			Err (e) => {
				println! ("Err: {:?}", e);
				backoff_delay = backoff_delay * 2 + 500;
				continue;
			},
			Ok (r) => {
				backoff_delay = 0;
				r
			},
		};
		
		if req_resp.status () != StatusCode::OK {
			continue;
		}
		
		println! ("Step 3");
		
		let body = req_resp.bytes ().await?;
		let body = String::from (std::str::from_utf8 (&body)?);
		
		println! ("Client requested {}", body);
		
		println! ("Step 4/5");
		let payload = String::from ("Ha ha hue hue it worked.\n");
		
		println! ("Step 6");
		let client = client.clone ();
		tokio::spawn (async move {
			let resp_req = Uri::builder ()
				.scheme ("http")
				.authority ("127.0.0.1:4000")
				.path_and_query ("/listen/alien_wildlands")
			.build ().unwrap ();
			
			let (tx, rx) = channel (2);
			//let rx: Receiver <Vec <u8>> = rx;
			
			tokio::spawn (async move {
				let path = "/home/user/pictures/bzqcChY.jpg";
				let mut f = File::open (path).await.unwrap ();
				let mut tx = tx;
				
				loop {
					let mut buffer = vec! [0u8; 256];
					let bytes_read = f.read (&mut buffer).await.unwrap ();
					
					buffer.truncate (bytes_read);
					
					if bytes_read == 0 {
						break;
					}
					
					tx.send (Ok::<_, Infallible> (buffer)).await;
				}
			});
			
			let resp_req = client.post (&format! ("http://127.0.0.1:4000/http_response/{}", body)).body (Body::wrap_stream (rx));
			
			println! ("Step 6");
			match resp_req.send ().await {
				Err (e) => {
					println! ("Err: {:?}", e);
				},
				Ok (_) => (),
			}
		});
	}
}
